@@ -1,31 +1,48 @@
 ########################################################################
 # Object::Trampoline
 # delay construction of objects until they are needed.
-#
-# the only purpose for the top two classes is having an autoload
-# that blesses things into O::T::Bounce.
-########################################################################
-########################################################################
-# housekeeping
 ########################################################################
 
-package Object::Trampoline  v1.30.2;
-use v5.24;
+package Object::Trampoline;
+
+use strict;
+
+our $VERSION = "1.01";
 
 use Carp;
 
+our $AUTOLOAD = '';
+
 ########################################################################
 # package variables
 ########################################################################
 
+my $use_class = 0;
+
 ########################################################################
-# AUTOLOAD is the only public interface
-########################################################################
-########################################################################
+# import passes in the switch as to whether the class is use-ed
+# or not.
 #
-# start by grabbing the destination class and its arguments
-# off the stack.  the constructor name is whatever is
-# being autoloaded.
+# note that the flag is universal: there is no good way to 
+# base it on the caller since there is noguarantee that the
+# eventual calling class will match the one seen here.
+
+sub import
+{
+    # discard the class.
+
+    shift;
+
+    my %argz = map { $_ => 1 } @_;
+
+    $use_class = $argz{ use_class } || 0;
+
+    0
+}
+
+########################################################################
+# the only purpose for this class is having an autoload
+# that blesses things into O::T::Bounce.
 #
 # there may not be any arguments... either way i need
 # to make a lexical copy of the stack for use in the
@@ -36,222 +53,110 @@ use Carp;
 #
 # $sub is syntatic sugar but is inexpensive enough to
 # construct.
-#
-# Note: there are no DESTROY blocks in the constructing
-# classes since no objects ever live there: they begin
-# life in O::T::Bounce.
-
-our $AUTOLOAD = '';
 
 AUTOLOAD
 {
-    # discard this class: once here it is used up.
-    # lacking a prototype is fatal; anything else becomes
-    # a run-time error.
+    # discard this class, then grab the "real" class
+    # and its arguments and the constructor's name.
 
-    my ( undef, $proto, @argz ) = @_;
+    shift;
 
-    $proto
-    or croak "Object::Trampoline: false prototype.";
+    my $class = shift
+    or croak "Bogus Object::Trampoline: missing destination class";
 
-    my $method  = ( split /::/, "$AUTOLOAD" )[ -1 ];
+    my @argz = @_;
 
-    my $sub     = sub { $proto->$method( @argz ) };
+    my $name = ( split /::/, $AUTOLOAD )[ -1 ];
 
-    bless $sub, 'Object::Trampoline::Bounce'
-}
-
-########################################################################
-# same gizmo as O::T except that the class is use-ed before the 
-# constructor is called. 
-########################################################################
-# housekeeping
-########################################################################
-
-package Object::Trampoline::Use;
-use v5.24;
-
-use Carp;
-
-########################################################################
-# package variables
-########################################################################
-
-*VERSION = \$Object::Trampoline::VERSION;
-
-########################################################################
-# AUTOLOAD is the only public interface
-########################################################################
-    
-our $AUTOLOAD = '';
-
-AUTOLOAD
-{
-    # this version does slightly more work since it 
-    # has to put using the module into the caller's
-    # class before calling the constructor.
-
-    my ( undef, $proto, @argz ) = @_;
-
-    $proto
-    or croak "Object::Trampoline::Use: false prototype.";
-
-    my $method  = ( split /::/, $AUTOLOAD )[ -1 ];
-    my $caller  = caller;
-
-    my $init
-    = qq
+    my $sub
+    = sub
     {
-        package $caller;
-        use $proto
-    };
-
-    my $sub =
-    sub
-    {
-        eval "$init"
-        or croak "Failed: $init\n$@";
+        eval "use $class" if $use_class;
         
-        $proto->$method( @argz )
+        $class->$name( @argz )
     };
 
     bless $sub, 'Object::Trampoline::Bounce'
 }
 
-########################################################################
-# where the object ends up. All this does is possibly use the package
-# then construct the object and dispatch the call to whatever the 
-# caller was looking for -- which may fail if the package doesn't
-# implement the method.
-#
-# $_[0] = $_[0]->() replaces the trampoline argument
-# with the real thing by calling its constructor -- call
-# by reference is a Very Good Thing.
-#
-# after that it can be shifted off and used to access
-# the method. note that this is necessary in order
-# to allow for classes which implement their methods
-# via AUTOLOAD (which will defeat using $obj->can( $name )).
-#
-# note that it's up to the caller to deal with any exceptions
-# that come out of calling the method.
-#
-# goto is a more effecient way to get there if the class
-# has an explicit method for handling the call; otherwise
-# use the name to dispatch the call.
-
 package Object::Trampoline::Bounce;
 
-use v5.24;
-
-use Carp;
-
-use Scalar::Util    qw( blessed         );
-use Symbol          qw( qualify_to_ref  );
-
-# version is defined by the main package.
+use strict;
 
 *VERSION = \$Object::Trampoline::VERSION;
 
+use Carp;
+
 our $AUTOLOAD = '';
 
 AUTOLOAD
 {
-    # caller gets back any execption as-is.
+    # replace the trampoline argument with the real
+    # thing by calling its constructor -- call by 
+    # reference is a Very Good Thing.
+    #
+    # after that it can be shifted off and used to 
+    # access the method. note that this is necessary
+    # in order to allow for classes which implement
+    # their methods via AUTOLOAD (which will defeat
+    # using $obj->can( $name )).
 
-    $_[0]   = $_[0]->()
-    or croak "Failed constructor";
+    $_[0] = $_[0]->();
 
-    my $class   = blessed $_[0]
-    or croak "Failed constructor: '$_[0]' not blessed";
+    my $class = ref $_[0];
 
-    my $method = ( split /::/, $AUTOLOAD )[ -1 ];
+    my $name = ( split /::/, $AUTOLOAD )[ -1 ];
 
-    if( my $sub = $class->can( $method ) )
+    # note that it's up to the caller to deal
+    # with any exceptions that come out of 
+    # calling the method.
+
+    if( my $sub = $_[0]->can( $name ) )
     {
+        # more effecient way to get there if the 
+        # sub has a real name...
+
         goto &$sub
     }
     else
     {
-        # deal with autoloaded methods, or die trying...
+        # ... otherwise go for it by name and let
+        # Perl resolve where the thing goes.
 
         my $obj = shift;
 
-        $obj->$method( @_ )
+        $obj->$name( @_ )
     }
 }
 
-# re-route methods from UNIVERSAL through the bounce.
-# allows $trampoline->VERSION to do the right thing.
-
-our $is_override
-= sub
-{
-    # sub allows testing w/o reproducing the 
-    # sanity checks in every test.
-    #
-    # basic checks: non-empty name w/o non-word 
-    # chars that has as a coderef in UNIVERSAL.
-
-    my $name    = shift
-    or return;
-
-    $name       =~ /\W/
-    and return;
-
-    defined &{ "UNIVERSAL::$name" }
-    or return;
-
-    1
-};
-
-for my $name ( keys %{ $::{ 'UNIVERSAL::' } } )
-{
-    # skip stash entries which cannot map to
-    # valid method names.
-
-    $is_override->( $name )
-    and
-    *{ qualify_to_ref $name }
-    = sub
-    {
-        $AUTOLOAD   = $name;
-        goto &AUTOLOAD
-    };
-}
-
-# stub destroy dodges AUTOLOAD for unused trampolines.
+# stub destroy is necessary to dodge AUTOLOAD for
+# unused objects.
 
 DESTROY {}
 
 # keep require happy
+
 1
+
 __END__
 
 =head1 NAME
 
-Object::Trampoline - delay object construction, and optinally
-using the class' module, until a method is actually dispatched,
-simplifies runtime definition of handler classes.
+Object::Trampoline - delay object construction until
+a method is actually dispatched, simplifies runtime definition
+of handler classes.
 
 =head1 SYNOPSIS
 
-    use Object::Trampoline;
-
     # the real class name is added to the normal constructor
     # and 'Object::Trampoline' used instead. the destination
     # class' constructor is called when object is actually 
     # used for something.
-    #
-    # The database handle is what you'd normally expect, but
-    # the statement handle is a trampoline: it gets constructed 
-    # on the first call via $sth->....
 
-    my $dbh = DBI->connect( ... );
-    my $sth = Object::Trampoline->prepare
-    (
-        $dbh, 'select foo from bar'
-    );
+    my $dbh = Object::Trampoline->connect( 'DBI', $dsn, $user, $pass, $conf );
+
+    my $sth = $dbh->prepare( 'select foo from bar' );
+
 
     # or specify the package and args from a config file
     # or via inherited data.
@@ -261,10 +166,10 @@ simplifies runtime definition of handler classes.
 
     my %config = Config->read( $config_file_path );
 
-    my ( $class, $const, @argz )
+    my ( $class, $const, $argz )
     = @config{ qw( class const args ) };
 
-    my $handle = Object::Trampoline->$const( $class, @argz );
+    my $handle = Object::Trampoline->$const( $class, $argz );
 
     # at this point ref $handle is 'Object::Trampoline::Bounce'.
 
@@ -272,18 +177,6 @@ simplifies runtime definition of handler classes.
 
     # at this point ref $handle is $class 
 
-    # there are times when it is helpful to delay using
-    # the object's class module until the object is 
-    # instantiated. O::T::U adds the caller's package
-    # and a "use $class" before the constructor.
-
-    my $lazy = Object::Trampoline::Use->frobnicate( $class, @stuff );
-
-    my $result = $lazy->susan( 'spin_me' );
-
-    # Note: isa and can are overloaded. Calling $lazy->isa will
-    # convert the object and return the corret type.
-
 =head1 DESCRIPTION
 
 There are times when constructing an object is expensive
@@ -308,20 +201,26 @@ Object::Trampoline uses whatever constructor the destination
 class calls (e.g., 'connect' for DBI) with the destination class
 is passed as the first argument.
 
-For example the normal DBI statement handle construcion becomes:
+For example the normal DBI construcion:
 
     my $dbh = DBI->connect( $dsn, $user, $pass, $conf );
 
-    my $sth = Object::Trampoline->prepare
-    (
-        "select ..."
-    );
+becomes:
 
-    # at this point blessed $sth is "Object::Trampline::Bounce"
+    my $dbh = Object::Trampoline->connect( 'DBI', $dsn, $user, $pass, $conf );
+
+eventually follwed by some use of the $dbh:
+
+    # at this point ref $dbh is "Object::Trampline::Bounce"
 
     my $sth = $dbh->prepare( 'select foo from bar' );
 
-    # at this point ref $dbh the statement handle.
+    # at this point ref $dbh is "DBI::db"
+
+This can be handy for error or other special event handlers
+they are not always used -- especially if they have to read
+initialization files or make database/directory service 
+connections to get their setup data.
 
 =head2 Runtime classes
 
@@ -331,8 +230,8 @@ class is passed as an argument. If various handler
 classes share a constructor name then the first argument
 to Object::Trampoline can be determined at runtime:
 
-    my $mailclass = $cmdline->{ mailer }        || 'SMTP::Simple';
-    my $mailconst = $cmdline->{ constructor }   || 'constructify';
+    my $mailclass = $cmdline->{ mailer } || 'SMTP::Simple';
+    my $mailconst = $cmdline->{ constructor } || 'constructify';
 
     ...
 
@@ -475,40 +374,6 @@ issues. Obviously at some point there may be a resource
 collision, but at least this delays things until the
 last possible time.
 
-=head2 Avoiding "circular use" situations
-
-When multiple layers of inheritence are used to build 
-up the metadata for an object there are times when 
-the layering gets complicated. It may be that all of
-the data is where it needs to be by the time objects
-are created, but the "use" pragma is in an implicit
-BEGIN block, which can cause startup errors.
-
-Object::Trampoline::Use delays using the constructing
-class until the object is actually used. This allows
-any configuration to be handled in time, even if some
-of the configuration involves objects from the included
-classes.
-
-Say that a project's Defaults.pm reads the command
-line and folds in static default values. After that
-Channels.pm creates various channel object using
-the configuration values. Another overall configuration
-module might then use the Defaults and Channels to
-get its list of exported values.
-
-Problems start when modules implementing the channels 
-use values from the configuration module, leading 
-to a circular use situation at startup.
-
-Object::Trampoline::Use avoids this issue by allowing
-the channel objects to be configured in lower-level
-modules, with their class import and instantiation 
-delayed until the objects are used. The configuration
-module can merrily hand out channel objects, who'se 
-classes will not be use-ed until after the configuration
-stack is complete.
-
 =head1 KNOWN BUGS
 
 =over 4
@@ -516,41 +381,28 @@ stack is complete.
 =item
 
 Not a bug, really, but if your constructor has side effects
-(e.g., opening log files) then delaying the construction
-will delay the side effects. Net result is that the side
-effects may have to migrate into the import where feasable
-or you just have to wait for the side effects to show
-up when the object is really used.
+(e.g., opening log files) then delaying the construction will
+delay the side effects. Net result is that the side effects
+may have to migrate into the import where feasable or you just
+have to wait for the side effects to show up when the object
+is really used.
 
 =item 
 
-Object::Trampoline does not use any classes for itself.
-This puts the onus of use, require, or do of the module
-defining the class onto the caller.
-
-Object::Trampoline::Use will use the module in the 
-caller's package, but the use will be delayed until
-an object is use-ed. This will delay side effects of
-import calls (see previous item).
-
-=item
-
-There is no way to pass arguments to the use call for
-a class in Object::Trampoline::Use. If that is necessary
-then either use the class with Object::Trampoline or 
-write a wrapper class whose constructor uses the class
-with the appropriate arguments.
-
-=item
-
-If your code depends on side-effects of the constructor
-manipulating the values in @_ then this module will not
-work for you: the call stack is copied to a lexical in 
-order to create the constructor closure. This will not
-be a problem for any module I know of.
+Also not really a bug, but it is the caller's responsability
+to actually "use" or "require" the destination class prior
+to actually constructing the object. The simple cases could
+be handled with a string eval, but then there isn't a good
+way to determine if a require or use is the proper choice.
+In the interest of simplicity I've left that to the caller.
 
 =over 4
 
+One way around this would be passing in a closure as the 
+first argument instead of the class. This could be executed
+as-is to get the object. If anyone has a strong opinion on
+this please warn me.
+
 =back
 
 =back
